import pygame, sys, os, random
from pygame.locals import *

#Game Constants
WINDOWHEIGHT = 800
WINDOWWIDTH = 800
MOVE = 4

SIZE = 10
FPS = 200

#Game variables
player_pos = (WINDOWWIDTH/2,WINDOWHEIGHT/2)

up = False
down = False
right = False
left = False

#function to load images files return image file and rect
#Arg1 name, Arg2 transparency value
def load_image(name, colourkey=None):
    fullname = os.path.join('Images',name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', name
        raise SystemExit, message
    if colourkey is not None:
        if colourkey is -1:
            colourkey = image.get_at((0,0))
        image.set_colourkey(colourkey, RLEACCEL)
    return image, image.get_rect()

#function to load sound files returns sound file
def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer:
        return NoneSound()
    fullname = os.path.join('Sounds',name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print 'Cannot load sound:', name
        raise SystemExit, messgae
    return sound

#adds vectors
def add_vector(vector1,vector2):
    return (vector1[0] + vector2[0] , vector1[1] + vector2[1])

#calculates position based upon angle and length
def angle_length(angle, length):
    radian_angle = math.radians(angle)
    return (-(length * math.sin (radian_angle)),
            length * math.cos (radian_angle))

#Exits window
def terminate():
    pygame.quit()
    sys.exit()

#updates player on screen
def surface_update(image,rect):
    windowSurface.blit(image,rect)

#changes the position and/or size of a rect
def move_rect(pos,size):
    return Rect(pos[0],pos[1],size[0],size[1])

#Checks to see if player exits window (escape or close button)
def check_quit(event):
    if event.type == QUIT:
        terminate()
    if event.type == KEYDOWN:
        if event.key == K_ESCAPE:
            terminate()

#Checks if player will move out of boundaries and returns True if so
def check_boundary(pos,vector):
    global SIZE, WINDOWHEIGHT, WINDOWWIDTH
    if pos[0]+vector[0] < 0:
        return True
    if pos[0]+SIZE+vector[0] > WINDOWWIDTH:
        return True
    if pos[1]+vector[1] < 0:
        return True
    if pos[1]+SIZE+vector[1] > WINDOWHEIGHT:
        return True
    return False

#Returns true if rects overlap
def check_collision(rect1, rect2):
    return rect1.colliderect(rect2)

#moves player based on input
def move_player():
    global player_pos, platformlist
    x = 0
    y = 0
    if up: y += -MOVE
    if down: y += MOVE
    if right: x += MOVE
    if left: x += -MOVE
    vector = (x,y)
    playerrect = Rect(player_pos[0]+x,player_pos[1]+y,SIZE,SIZE)
    if check_boundary(player_pos,vector):
        return
    for platform in platformlist:
        if check_collision(playerrect, platform[1]):
            return
    player_pos = add_vector(player_pos,vector)

#Check for input and moves player
def check_move(event):
    global up, down, left, right
    if event.type == KEYUP:
        if event.key == K_UP: up = False
        if event.key == K_DOWN: down = False
        if event.key == K_RIGHT: right = False
        if event.key == K_LEFT: left = False
    if event.type == KEYDOWN:
        if event.key == K_UP: up = True
        if event.key == K_DOWN: down = True
        if event.key == K_RIGHT: right = True
        if event.key == K_LEFT: left = True
    
#Other functions

#set up pygame and window
pygame.init()
mainClock = pygame.time.Clock()
windowSurface = pygame.display.set_mode((WINDOWWIDTH,WINDOWHEIGHT))
pygame.display.set_caption('Our Game Needs a Title')

#load images
tester_image, tester_rect = load_image('tester.png')
tester_rect = move_rect(player_pos,(SIZE,SIZE))

platformimage, platformrect = load_image('platform.png')
platformlist = [[platformimage,(500,600,100,10)],
                [platformimage,(0,400,100,10)],
                [platformimage,(200,500,100,10)],
                [platformimage,(500,700,100,10)],
                [platformimage,(400,500,100,10)],
                [platformimage,(500,200,100,10)],
                [platformimage,(300,400,100,10)]]

background = load_image('background.png')[0]

#Event loop, manages controls and updates images etc.
windowSurface.blit(background, (0,0))
surface_update(tester_image,tester_rect)
for platform in platformlist:
    surface_update(platform[0],platform[1])
pygame.display.update()

while True:
    for event in pygame.event.get():
        check_quit(event)
        check_move(event)
    move_player()
    tester_rect = move_rect(player_pos,(SIZE,SIZE))
    
    windowSurface.blit(background, (0,0))
    surface_update(tester_image,tester_rect)
    for platform in platformlist:
        surface_update(platform[0],platform[1])
    pygame.display.update()
    mainClock.tick(FPS)
